#include <string.h>
#include <mex.h>
#include "blosc.h"


#define INPUT prhs[0]
#define SIZE prhs[1]
#define OUTPUT plhs[0]


void mexFunction (int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray*prhs[])

{
  if (!(nrhs == 2 && nlhs == 1)) 
    mexErrMsgTxt("Need 2 input & 1 output"); 
  
  if(!(mxGetClassID(INPUT) == mxUINT8_CLASS && !mxIsSparse(INPUT) && !mxIsComplex(INPUT)))
    mexErrMsgTxt("Need dense uint8 array as input.");

  if(!(mxGetClassID(SIZE) == mxDOUBLE_CLASS && !mxIsSparse(SIZE) && !mxIsComplex(SIZE) && 
       mxGetNumberOfElements(SIZE) == 2))
    mexErrMsgTxt("2nd argument is 2 element size");
    
  const double *size2 = mxGetPr(SIZE);
  int n = size2[0], m = size2[1];

  size_t size = mxGetNumberOfElements(INPUT); 
  
  OUTPUT = mxCreateNumericMatrix (n, m, mxUINT8_CLASS, mxREAL);  

  /* asking for shuffle with elementsize = 1 does not work normally */
  int decompressed_size = blosc_decompress(mxGetData(INPUT), mxGetData(OUTPUT), n * m); 
  
  mxAssert(decompressed_size > 0, "blosc_decompress failed"); 

}
