% Manages a set of descriptors stored in flexdata format, compressed
% with PQ and Blosc.
%   
% The first argument is a command defining the operation to perform. 
% 
%     descriptor_cache_mex('load', basedirs, imagemask, use_mmap)
% 
% Loads the Flexdata descriptors of n channels from a set of n basedirs
% (cell of size n with strings). imagemask (logical array of size n)
% indicates which channels are full-image channels, as opposed to box
% descriptors. use_mmap specifies whether to mmap the descriptors or to
% read them to RAM.
% 
%     descriptor_cache_mex('load-servers', basedirs, imagemask, servers)
% 
% Does the same, but the descriptors are loaded on remote servers,
% specified by the struct array servers. The struct contains:
% 
%    servers(i).name: hostname of the server machine 
%    servers(i).port: network port 
%    servers(i).last_image: server i will load image numbers 
% 
%        servers(i-1).last_image + 1 : servers(i).last_image 
% 
% To get the PQ codes for a selection of n boxes:
% 
%    [npts, codes] = descriptor_cache_mex('catpq', idx, from_bb)
% 
% idx is a n-by-2 uint32 array, such that the i'th extracted box is box
% idx(i, 2) on image idx(i, 1), 0-based. Returns the number of points
% (weighted) as nchannel-by-n array npts, and the codes in the columns
% of the codes array. from_bb is a boolean indicating whether we want
% codes for ground-truth boxes or candidate boxes
% 
% Compute the detection scores for the boxes for a set of images:
% 
%    [scores, npts_stats] = descriptor_cache_mex('detect', idx, alls_pq, l2sq)
% 
% idx is a uint32 array of images we want scores for (0 based). alls_pq
% is a 256-by-nsq array of classifier weights, and l2sq, same size,
% contains the corresponding L2 normalization coefficients. the returned
% scores are in a column vector. npts_stats indicates, for each score,
% the sum and maximum number of points in the corresponding box, over
% channels.

error('compile the mex file')
