

#ifndef FLEXDATA_H_INCLUDED
#define FLEXDATA_H_INCLUDED


typedef unsigned long uint64; 
typedef unsigned char uint8; 

typedef struct {
  int nimage;             /* nb of images */
  int nsq;                /* decoded dimension */ 
  uint64 totbox;          /* total nb of boxes */
  uint64 *nbox;           /* nb of boxes per image */    
  uint64 *cum_nbox;       /* same, cumulative */    
  float *npts;            /* nb of pts per box (weighted, hence the float) */
  uint64 *blosc_offsets;  /* beginning of each image in blosc block */
  uint8 *codes;           /* blosc-encoded data */

  int fd1, fd2;           /* file descriptor if codes are memory mapped (else 0) */
  long sz1, sz2;          /* also for mmap */
} flexdata_block_t;


flexdata_block_t *flexdata_block_new(const char *dirname, const char *fieldname, int do_mmap); 

void flexdata_block_delete(flexdata_block_t *b);


typedef struct {
  flexdata_block_t *bb;   /* descriptors of the candidate boxes */
  flexdata_block_t *gt;   /* descriptors of the ground-truth boxes */
  flexdata_block_t *im;   /* descriptors of the whole image (exclusive with bb and gt) */
} flexdata_channel_t; 

typedef struct {
  int nimage;             /* total nb of images */
  int nchannel;           /* total nb of channels */
  int tot_nsq;            /* sum of nsq of channels */
  uint64 *nbox, *cum_nbox; /* from one of the channels */
  flexdata_channel_t *channels; 
} flexdata_t; 

flexdata_t *flexdata_new(int nchannel, 
                         const char **dirnames, 
                         const int *immmasks, 
                         int do_mmap); 
                         
void flexdata_delete(flexdata_t *fd); 

/* sample a set of descriptors in pq-coded representation */
int flexdata_sample_descriptors(flexdata_t * fd,
                                 int nsample,        /* nb of samples we want descriptors for */
                                 int from_bb,        /* bb or gt */
                                 const int *imind,   /* from what images (size nsample) */
                                 const int *bbind,   /* from what bounding boxes (size nsample) */
                                 float *npts,        /* size (nchannel, nsample) */
                                 uint8 *pqcodes);    /* size (nsq * nchannel, nsample) */


/* compute scores for all bb's on a set of images */
int flexdata_compute_bb_scores(const flexdata_t * fd,
                                int nimage,               /* nb of images we want scores for */
                                const int *imind,         /* indices of images we want to handle */
                                const double *alls_pq,    /* coefficients for numerators */
                                const double *l2normsq,   /* coefficients for per-cell normalization */
                                double *scores,           /* output scores, size (nb of bboxes per image) */
                                double *npts_stats);      /* per-box statistics on nb of points */

int flexdata_count_nbox(int nimage,
                        const uint64 * nbox,                         
                        int ni,                           /* nb of images in imind */
                        const int *imind);                /* indices in nbox table */

#endif



