function [f_xynorm, d_cols] = extract_features(imname, desc, sepscales)
  
% adapted from Gokberk's fv_locfeat_load > helper_objdet
  
  mirror = false;
  
  im = imread(imname);
  
  % params from Gokberk's fv_locfeat_tool_objdet
  pfeat.sz     = 12;        % height=width of the smallest patch.
  pfeat.nsc    = 16;        % number of scales, better than defining min/max size since this is unique
  pfeat.scstep = 1.2;
  pfeat.rstep  = .25;       % step = sz * rstep
  pfeat.desc   = desc;   % sift | color

  switch(desc)
   case 'sift'
    im = cn_rgb2gray(cn_im2single(im));
    pfeat.magnif = 6;     % phow:6      image is smoothed with kernel sigmal=sz/magnif.
    pfeat.wndsz  = 1.5;   % phow:1.5    gaussian windowing size.
    pfeat.contth = 0.005; % phow:0.005  constrast threshold
    
   case 'color'
    im = cn_im2double(im);
    if size(im,1)==1
      im = repmat(im,[1 1 3]);
    end
   otherwise
    error('unnkown');
  end
  
  if mirror
    im = im(:,end:-1:1,:);
  end
  imsize = [size(im,1) size(im,2)];
  
  
  assert(mod(pfeat.sz,4)==0);
  binsize = pfeat.sz/4;
  step = round(pfeat.sz * pfeat.rstep);
  
  imscales = (1 / pfeat.scstep) .^ (0 : pfeat.nsc - 1);
  
  pyr = cn_impyramid(im,'cvCubic/cvArea',imscales);
  pyr_f = cell(1,length(pyr));
  pyr_d = cell(1,length(pyr));

  switch(pfeat.desc)
      
   case 'sift'
    dsiftopt = {'size',binsize,'step',step,'fast','windowsize',pfeat.wndsz,'norm','FloatDescriptors'};
    for sci = 1:length(pyr)
      curimsz = size(pyr{sci});
      
      % make local descriptors centered in the image. normally, this does not change number of patches
      % but due to contrast threshold (contth), result may differ, which is fine.
      curbounds = cn_vldsifthelper('centralize',struct2('imsz',curimsz,'step',step,'size',binsize));
      [f,d] = vl_dsift(pyr{sci},'bounds',curbounds,dsiftopt{:});
      
      % drop empty features
      % todo: return empty feature positions
      m = f(3,:) >= pfeat.contth;
      allf{sci} = bsxfun(@rdivide,double(f(1:2,m))-1,curimsz([2 1])'); % normalize to [0,1], ignore the small,consistent bug here. normally we should normalize by "curimsz([2 1])-1".
      alld{sci} = double(d(:,m)); % descriptors are in range [0,255] but I don't see a reason to scale them back.
    end
   
   
   
   case 'color'
    for sci = 1:length(pyr)
      curimsz = [size(pyr{sci},1) size(pyr{sci},2)];
      
      % make local descriptors centered in the image. normally, this does not change number of patches
      % but due to contrast threshold (contth), result may differ, which is fine.
      curbounds = cn_vldsifthelper('centralize',struct2('imsz',curimsz,'step',step,'size',binsize));
      curbb = cn_vldsifthelper('intboxes',struct2('imsz',curimsz,'step',step,'size',binsize,'bounds',curbounds));
      f = [(curbb(1,:)+curbb(3,:))/2; (curbb(2,:)+curbb(4,:))/2];
      d = cn_localcolordesc(pyr{sci},curbb,struct2('dtrans','none','grid',[4 4]));
      
      allf{sci} = bsxfun(@rdivide,double(f)-1,curimsz([2 1])'); % normalize to [0,1], ignore the small,consistent bug here. normally we should normalize by "curimsz([2 1])-1".
      alld{sci} = double(d); % descriptors are in rnage [0,255] but I don't see a reason to scale them back.
    end
   
   
   otherwise
    error('unknown desc');
  end
  
  if sepscales 
    f_xynorm = allf;
    d_cols = alld;
  else  
    f_xynorm = [allf{:}];
    d_cols = [alld{:}];
  end
end
