

function data = get_sample_descriptors(descriptors, samples, box_type);
  % box_type == 0: ground truth (also uses orgbbind)
  % == 1: candidate boxes selected by orgbbind
  % == 2: all candidate boxes  
  pq0 = descriptors.pqs(1);
  nchan = length(descriptors.channels);   

  if length(samples) == 0
    data.codes = zeros(pq0.nsq * nchan, 0, 'uint8');
    data.npts = zeros(nchan, 0); 
    data.wscr = zeros(1, 0); 
    return
  end

  if descriptors.cache_level == 0
    % was loaded in Matlab
    data = get_sample_descriptors_0(descriptors, samples, box_type);
  else
    % was loaded in mex function
    data = get_sample_descriptors_1(descriptors, samples, box_type);
  end
  
  data = preprocess_data(descriptors, samples, data);
end

  
function data = get_sample_descriptors_0(descriptors, samples, box_type)

  pq0 = descriptors.pqs(1);
  nchan = length(descriptors.channels);   

  
  nsamples = size(vertcat(samples.bbuint8), 1);

  nsamples = size(vertcat(samples.bbuint8), 1);
  data.codes = zeros(pq0.nsq, nchan, nsamples, 'uint8');
  data.npts = nan(nchan, nsamples);

  outi = 0; 
  for sami = 1:length(samples)
    sample = samples(sami); 
    imi = sample.orgimi; 
    
    if box_type == 2
      ndesc = size(sample.bbuint8, 1);
    else
      ndesc = length(sample.orgbbind);       
    end      
    
    for chani = 1:nchan
      channel = descriptors.channels{chani}; 
      
      if channel.isfullim        
        % this is a context channel with the full image descriptor         
        r = channel.im(imi); 
        data.npts(chani, outi + 1 : outi + ndesc) = repmat(r.npts, 1, ndesc); 
        data.codes(:, chani, outi + 1 : outi + ndesc) = repmat(r.codes, 1, ndesc); 
      else        
        if box_type == 0
          % we were asked for ground thruth boxes
          r = channel.gt(imi);
        else
          % we want candidate boxes
          r = channel.bb(imi);
        end
        all_codes = r.codes; 
        all_npt = r.npts;        
        if box_type ~= 2
          all_npt = all_npt(sample.orgbbind); 
          all_codes = all_codes(:, sample.orgbbind); 
        end
        
        data.npts(chani, outi + 1 : outi + ndesc) = all_npt;
        data.codes(:, chani, outi + 1 : outi + ndesc) = all_codes;
      end
      
    end    
    outi = outi + ndesc;
  end  
  
end

function data = get_sample_descriptors_1(descriptors, samples, box_type)
  
  pq0 = descriptors.pqs(1);
  nchan = length(descriptors.channels);   
  nsamples = size(vertcat(samples.bbuint8), 1);
  idxtab = zeros(nsamples, 2, 'int32'); % one column for image indices, one for box indices
  di = 0;
  for sami = 1:length(samples)
    s = samples(sami); 
    n = size(s.bbuint8, 1);
    if n == 0
      continue
    end
    idxtab(di + 1: di + n, 1) = s.orgimi - 1;
    if box_type == 0 || box_type == 1
      idxtab(di + 1: di + n, 2) = s.orgbbind - 1;
    else
      idxtab(di + 1: di + n, 2) = (1:n) - 1;
    end
    di = di + n; 
  end
  from_bb = (box_type ~= 0);

  [data.npts, data.codes] = descriptor_cache_mex('catpq', idxtab, from_bb); 
  
end



function data = preprocess_data(descriptors, samples, data)
  % adapted from fv_objdet_pqfvmog_prepcatdata
  nchan = length(descriptors.channels);   
  nsq = descriptors.pqs(1).nsq;
    
  data.orgimind = cell(1,length(samples));
  for i = 1:length(samples)
    ndesc = size(samples(i).bbuint8, 1);
    data.orgimind{i} = repmat(samples(i).orgimi, 1, ndesc);
  end
  data.orgimind = [data.orgimind{:}];

    
  pointmask = logical(zeros(nchan, 1));
  for i = 1:nchan
    pointmask(i) = descriptors.channels{i}.ismask;
  end  
  
  l2normsq_c = {};
  for i = 1:length(descriptors.pqs)
    l2normsq_c{i} = descriptors.pqs(i).l2normsq;
  end

  data.codes = reshape(data.codes, nsq * nchan, []);
  
  ns = pq_l2normsq(data.codes, l2normsq_c, uint64(descriptors.z2c)); % (Z N)
  data.wscr = 1 ./ max(eps, realsqrt(ns));
  
  x = data.npts(pointmask, :); 
  data.sumofnpts = sum( x , 1);
  data.maxofnpts = max( x , [], 1);
  
  
end
