function centers = cn_kmeansparallel(p)
% Do k-means on regular subdimensions of a data using multithreading.
% This is essentially in PQ training, though, may have other uses where several small-dimensional clustering is
% needed to be done.
%
% Implementation is very close to jakob_kmeans, gives usually equivalent results (except when there are boundary conditions
% in reinitializing the empty clusters). However, usually, cn_kmeansparallel is much faster for the PQ training purposes.
%
% If p.data is in single precision, calculations are still done in double precision, without consuming a lot of extra memory.
%
% INPUT
% p
%   .data       (N D)   Row-by-row data, allowing efficient chunking. [single or double]
%   .nsq        (1 1)   Number of sub-quantizers s.t. mod(D,nsq)=0
%   .K          (1 1)   Number of clusters per subquantizer; up to intmax('uint32').
%   .randseed   (1 1)   Random seed. For the i-th subquantizer, (randseed+i) will be used where 1 <= i <= nsq. 
%   .maxiter    (1 1)   Maximum number of iterations for each clustering run. 0
%   .[centers]          Initial cluster in the output format. [single or double] 
%                       If not provided, within each subquantizer, the following initialization is used (after randseed)
%                           Msqi = subdata(cn_randmask(N,K),:); 
%
% OUTPUT
% centers       (ds K nsq)    Note that within each cluster, centers are col-by-col. [always double]
%
% R.G.Cinbis, Sep 2012

% all is handled within c.
centers = cn_kmeansparallel_mex(p);

