#include "cn_blas.hpp"

/**
 * Author: R. Gokberk Cinbis.
 */

#ifndef CN_NOBLAS


void cn::blas::_::mtimes_lowdim(const char &opX, const char &opY, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N, ptrdiff_t *ldX, ptrdiff_t *ldY) {

    if ( opX=='N' )
        *ldX=M;
    else if (opX=='T')
        *ldX=D;
    else
        throw std::runtime_error("Invalid opX");

    if ( opY=='N' )
        *ldY=D;
    else if (opY=='T')
        *ldY=N;
    else
        throw std::runtime_error("Invalid opX");

}

#if !defined(_WIN32) && !defined(dgemm)
#define dgemm dgemm_
#endif

#if !defined(_WIN32) && !defined(sgemm)
#define sgemm sgemm_
#endif

void cn::blas::mtimes(const char opX, const float* X, const char opY, const float* Y, float *O, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N) {
    float one = 1.0, zero = 0.0;
    ptrdiff_t ldX,ldY;
    _::mtimes_lowdim(opX,opY,M,D,N,&ldX,&ldY);
    if ( M==0 || N==0 ) { return; } // output is empty array.
    memset(O,0,sizeof(double)*M*N); // mandatory since O is both an input and an output.
    if ( D==0 ) { return; }  // output is zeros. don't call blas in boundary cases it might fail.
    sgemm((char *) &opX, (char *) &opY, (ptrdiff_t *) &M, (ptrdiff_t *) &N, (ptrdiff_t *) &D, &one, 
            (float *) X, (ptrdiff_t *) &ldX, (float *) Y, (ptrdiff_t *) &ldY, &zero, O, (ptrdiff_t *) &M);
}

void cn::blas::mtimes(const char opX, const double* X, const char opY, const double* Y, double *O, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N) {
    double one = 1.0, zero = 0.0;
    ptrdiff_t ldX,ldY;
    _::mtimes_lowdim(opX,opY,M,D,N,&ldX,&ldY);
    if ( M==0 || N==0 ) { return; } // output is empty array.
    memset(O,0,sizeof(double)*M*N); // mandatory since O is both an input and an output.
    if ( D==0 ) { return; }  // output is zeros.
    dgemm((char *) &opX, (char *) &opY,(ptrdiff_t *)  &M, (ptrdiff_t *) &N, (ptrdiff_t *) &D, &one, 
            (double *) X, (ptrdiff_t *) &ldX,  (double *)Y, (ptrdiff_t *) &ldY, &zero, O, (ptrdiff_t *) &M);
}

// SEE http://www.netlib.org/clapack/cblas/saxpy.c
// SEE http://docs.oracle.com/cd/E19059-01/stud.9/817-6700/saxpy.html
/*
void cn::blas::axpy(const float a, const ptrdiff_t d, const float * x, float * y) {
}
*/





#else // CN_NOBLAS: Baseline implementation, truly naive implementation.

#error NOT TESTED YET.

template <class T>
void baseline_mtimes(const char opX, const T* X, const char opY, const T* Y, T *O, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N) {
    uint64_t stepx,stepy;
    bool tr_x = false;
    bool tr_y = false;
    if ( opX=='N' )
        stepx = M;
    else if (opX=='T')
        tr_x = true;
        stepx = 1;
    else
        throw std::runtime_error("Invalid opX");

    if ( opY=='N' )
        stepx = 1;
    else if (opY=='T')
        tr_y = true;
        stepy = N;
    else
        throw std::runtime_error("Invalid opX");

    for ( uint64_t m = 0 ; m < M ; ++m ) {
        const T *x;
        if ( !tr_x )
            x = X + m;
        else
            x = X + D * m;
        for ( uint64_t n = 0 ; n < N ; ++n ) {
            const T *y;
            if ( !tr_y  )
                y = Y + D * n;
            else
                y = Y + n;
            T o = 0;
            for ( uint64_t d = 0; d < D; ++d ) {
                o += x[m * stepx] + y[ m * stepy ];
            }
            O[n*M+m] = o;
        }
    }
}

void cn::blas::mtimes(const char opX, const float* X, const char opY, const float* Y, float *O, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N) {
    baseline_mtimes(opX,X,opY,YO,M,D,N);
}
void cn::blas::mtimes(const char opX, const double* X, const char opY, const double* Y, double *O, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N) {
    baseline_mtimes(opX,X,opY,YO,M,D,N);
}

#endif
