#pragma once

/**
 * Author: R. Gokberk Cinbis.
 */


/**
 * Contains wrappers over blas functions.
 */
#include <cstddef> // to get ptrdiff_t, size_t.
#include <stdint.h> // to get uint8_t
#include <cstring>
#include <stdexcept> // runtime_error
#include "blas.h"

namespace cn {
    namespace blas {
        namespace _ {
            
            /**
             * Helper for mtimes()
             */
            void mtimes_lowdim(const char &opX, const char &opY, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N, ptrdiff_t *ldX, ptrdiff_t *ldY);
        }

        /** 
         * Do matrix multiplication 
         *      O = op(X) * op(Y)
         *
         * ARGUMENTS
         * opX:     'N' | 'T'
         * X        op(X): (M D) , col-major.
         * opY      'N' | 'T'
         * Y        op(Y): (D N) , col-major.
         * O        (M N), preallocated. Initial data is ignored.
         * M        Scalar.
         * D        Scalar.
         * N        Scalar.
         *
         */
        void mtimes(const char opX, const float* X, const char opY, const float* Y, float *O, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N);
        void mtimes(const char opX, const double* X, const char opY, const double* Y, double *O, const ptrdiff_t M, const ptrdiff_t D, const ptrdiff_t N);

        /**
         * Do "y = a .* x + y" where a is a scalar, x and y are (d 1).
         *
         * ARGUMENTS
         * a        Scalar weight.
         * d        
         * x        (d 1)
         * y        (d 1)
         */
        // I found it rather unnecessary to use in my case. Since "d" was too small to bother calling blas.
        /*
        void axpy(const float a, const ptrdiff_t d, const float * x, float * y);
        void axpy(const double a, const ptrdiff_t d, const double * x, double * y);
        */

        
#include "private/cn_blas.templates.cpp"

    }
}


