#ifndef CN_OS
#define CN_OS

/**
 * Author: R. Gokberk Cinbis.
 */


// OS macros and definitions for cross-os portability.
//
// Automatically defines either CN_WINDOWS or CN_LINUX.
// Treats cygwin as linux environment.
// Guarantees that exactly one os indicator is defined.
// Also includes several functions from GNULIB.
//
// Use with
// #if defined(CN_LINUX)
// #elif defined(CN_WINDOWS)
// #else
// #error Unknown platform
// #endif
//
// SEE http://sourceforge.net/apps/mediawiki/predef/index.php?title=Operating_Systems#Windows
//
// R.G.Cinbis, July 2012

#if defined(CN_LINUX) || defined(__linux__)
#ifndef CN_LINUX
#define CN_LINUX
#endif
#ifndef CN_OSLINUX      // a subset of CN_LINUX machines: non-cygwin, true linux
#define CN_OSLINUX
#endif
#endif

#if defined(__CYGWIN__)
#ifndef CN_LINUX
#define CN_LINUX
#endif
#ifndef CN_CYGWIN
#define CN_CYGWIN
#endif
#endif

#if defined(CN_WINDOWS) || defined(__WIN32__) || defined(_WIN32) || defined(WIN32) || defined(__WINDOWS__) || defined(__TOS_WIN__)
#ifndef CN_WINDOWS
#define CN_WINDOWS
#endif
#endif

#if !defined(CN_LINUX) && !defined(CN_WINDOWS)
#error Unknown platform
#endif

#if defined(CN_LINUX) && defined(CN_WINDOWS)
#error Confusing platform indicator
#endif

// whether c++11 is supported, works on gcc only.
#if __cplusplus > 199711L
#define CN_CPP11
#endif

// CN_MEX is automatically set by cn_makeall.

#endif

