#include "poolobjdesc_init.hpp"

/**
 * Author: R. Gokberk Cinbis.
 */



mwSize maxN = max(allN) ;
mwSize maxthread = setnumparallel( R );
mwSize nscale = getnscale();
if (nscale == 0)
    throw runtime_error("nscale cannot be 0");

// dictionary is not needed here.
unsetDictionary();
D = getDapp();
K = getKapp();

shared_ptr<mask_t> tmpmasks(new mask_t[ maxN * maxthread ],DeleterArray<mask_t>);
string errstr;

#pragma omp parallel for
for (mwSize r = 0; r < R; ++r) { 
    try { 

        const mwSize threadi = getthreadindex(maxthread);

        mask_t *tmpm = tmpmasks.get() + threadi * maxN;
        npts_t obj_npts = 0; // number of points within the object
        double * obj_out = out + D * Kapp * r;

        // get scales
        int16_t minscale,maxscale;
        getMinMaxScaleForRegion( r, &minscale, &maxscale );
        for (int16_t sci = minscale; sci <= maxscale; sci+=scstep) {

            const double *Q = allQ[sci];
            const mwIndex *Q_Ir = allQ_Ir[sci];
            const mwIndex *Q_Jc = allQ_Jc[sci];

            // it is definite that we'll compare each point wrt current box at least once.
            // so it is better check beforehand which points are inside the box.
#if defined(POOL_BOXES) || defined(POOL_BOXES_WITH_PTWEIGHTS)
            // if setmasks() we may effectively be pooling segments here as well.  however, it is more flexible but
            // less efficient here as we allow overlapping segments given as per-box segmentation masks.
            obj_npts += getptmask4box(sci,r,tmpm);

#elif defined(POOL_TREE)
            // convert compact "points within the segment" list into a mask of size (1 curnpts), which is far
            // easier to handle below
            obj_npts += getptmask4segment(nscale,sci,r,tmpm);

#else
#error unknown
#endif

            const double *V = allV[sci]; // original descriptors

            double *obj_out_col = obj_out;
            for (mwSize k = 0; k < Kapp; ++k) {

                const double *xQ = Q + Q_Jc[k]; 
                mwSize xnnz = Q_Jc[k+1] - Q_Jc[k]; /* #nonzero in Q(:,k) */
                const mwIndex *xIr = Q_Ir + Q_Jc[k]; 

                for ( mwSize zi = 0; zi < xnnz; zi++ ) { // loop over nonzero elements in Q's current column
                    const mwSize &pti = *xIr;

#if HAS_PT_WEIGHTS
                    if (tmpm[pti] > FvGmm_PoolObjDesc_PtWeightEps) {  // check whether point is inside the box
                        double *xout = obj_out_col;
                        const double *xV = V + D * pti;
                        const double ptw = *xQ * tmpm[pti];
                        for (mwSize i = 0; i < D; ++i) {
                            (*xout++) += (*xV++) * ptw;
                        }
                    }
#else
                    if (tmpm[pti]) {  // check whether point is inside the box
                        double *xout = obj_out_col;
                        const double *xV = V + D * pti;
                        for (mwSize i = 0; i < D; ++i) {
                            (*xout++) += (*xV++) *  *xQ; 
                        }
                    }
#endif

                    ++xQ; ++xIr;  // next nonzero Q row
                }
                obj_out_col += D;

            } // loop over k
        } // loop over sci

        npts[r] = obj_npts; // set after counting over all scales.

    }
#include "poolobjdesc_catcherr.hpp"

}

#include "poolobjdesc_end.hpp"

