#include "poolobjdesc_init.hpp"

/**
 * Author: R. Gokberk Cinbis.
 */


if ( !fvnormalizer )
    throw runtime_error("dictionary is required for fisher vectors");
if (K != Kapp)
    throw runtime_error("For poolfv, K and Kapp should be the same");
if (D != Dapp)
    throw runtime_error("For poolfv, D and Dapp should be the same");

mwSize maxN = max(allN) ;
mwSize maxthread = setnumparallel( R );
mwSize nscale = getnscale();
if (nscale == 0)
    throw runtime_error("nscale cannot be 0");

// precompute [d_cols;d_cols.^2] if possible.
vector< shared_ptr<double> > allV2;
mwSize D2 = D * 2;
catWithSquares(&allV2);

shared_ptr<mask_t> tmpmasks(new mask_t[ maxN * maxthread ],DeleterArray<mask_t>);
string errstr;

#pragma omp parallel for
for (mwSize r = 0; r < R; ++r) { 
    try {

#ifdef POOL_TREE
        if ( skipifnonexist && (!segmentexists.get()[r]) )
            continue; 
#endif

        const mwSize threadi = getthreadindex(maxthread);

        mask_t *tmpm = tmpmasks.get() + threadi * maxN;
        npts_t obj_npts = 0; // number of points within the object
        double * obj_out = out + (1 + 2*D) * K * r;

        int16_t minscale,maxscale;
        getMinMaxScaleForRegion( r, &minscale, &maxscale );
        for (int16_t sci = minscale; sci <= maxscale; sci+=scstep) {

            const double *Q = allQ[sci];
            const mwIndex *Q_Ir = allQ_Ir[sci];
            const mwIndex *Q_Jc = allQ_Jc[sci];

            // it is definite that we'll compare each point wrt current box at least once.
            // so it is better check beforehand which points are inside the box.
#if defined(POOL_BOXES) || defined(POOL_BOXES_WITH_PTWEIGHTS)
            // if setmasks() we may effectively be pooling segments here as well.  however, it is more flexible but
            // less efficient here as we allow overlapping segments given as per-box segmentation masks.
            obj_npts += getptmask4box(sci,r,tmpm);
#elif defined(POOL_TREE)
            // convert compact "points within the segment" list into a mask of size (1 curnpts), which is far
            // easier to handle below
            obj_npts += getptmask4segment(nscale,sci,r,tmpm);
#else
#error unknown
#endif

            const double *V = allV2[sci].get(); // precomputed
            double *obj_out_col = obj_out;
            for (mwSize k = 0; k < K; ++k) {

                const double *xQ = Q + Q_Jc[k]; 
                mwSize xnnz = Q_Jc[k+1] - Q_Jc[k]; /* #nonzero in Q(:,k) */
                const mwIndex *xIr = Q_Ir + Q_Jc[k]; 

                for ( mwSize zi = 0; zi < xnnz; zi++ ) { // loop over nonzero elements in Q's current column
                    const mwSize &pti = *xIr;

#if HAS_PT_WEIGHTS
                    if (tmpm[pti] > FvGmm_PoolObjDesc_PtWeightEps) {  // check whether point is inside the box
                        const double ptw = *xQ * tmpm[pti];
                        double *xout = obj_out_col;
                        *(xout++) += ptw; // increment p(k|x)
                        const double *xV = V + D2 * pti;
                        for (mwSize i = 0; i < D2; ++i) {
                            (*xout++) += (*xV++) * ptw; // increment p(k|x)*x and p(k|x)*(x.^2)
                        }
                    }
#else
                    if (tmpm[pti]) {  // check whether point is inside the box
                        double *xout = obj_out_col;
                        *(xout++) += *xQ; // increment p(k|x)
                        const double *xV = V + D2 * pti;
                        for (mwSize i = 0; i < D2; ++i) {
                            (*xout++) += (*xV++) *  *xQ; // increment p(k|x)*x and p(k|x)*(x.^2)
                        }
                    }
#endif

                    ++xQ; ++xIr;  // next nonzero Q row
                } 
                obj_out_col += (D2 + 1);

            } // loop over k
        } // loop over sci

        npts[r] = obj_npts; // set after counting over all scales.

        // we've pooled over scales, now convert moments into fisher vectors with analytical approximation.
        // this operation can be done independently for each k
        stats2fv(obj_out,obj_npts,avgfv);

    } 
#include "poolobjdesc_catcherr.hpp"

}

#include "poolobjdesc_end.hpp"

