function [X Y areaX areaY NORM_OPT IS_PIXELCOORD] = cn_bboverlap_prepare(X,Y,option)
% Preprocessing function utilized by cn_bboverlap()
% In order to use bb-overlap defined in cn_bboverlap_functions.cpp, 
%
% INPUT
% X,Y,option:       See cn_bboverlap
%
% todo: already colwise input support.
% 
% OUTPUT
% X,Y               (m 4), (n 4) box coordinates.
% areaX,areaY       (m 1), (n 1) box areas, as required by cn_bboverlap_functions.cpp
% NORM_OPT          See cn_bboverlap_functions.cpp
% IS_PIXELCOORD     See cn_bboverlap_functions.cpp
%
% USED BY
% cn_bboverlap.m
% cn_nmsgreedy.m
%
% R.G.Cinbis, May 2012

if ~isa(X,'double') || ~isa(Y,'double')
    % annoying: disp('cn_bboverlap requires data in double, converting to double automatically');
    X = double(X);
    Y = double(Y);
end

if ~isempty(X)
    assert(size(X,2)==4);
else
    X = zeros(0,4);
end
if ~isempty(Y)
    assert(size(Y,2)==4);
else
    Y = zeros(0,4);
end

% handle negative-sized bb's:
w_n = false;
nmask = X(:,3) < X(:,1); tmp = X(nmask,3); X(nmask,3) = X(nmask,1); X(nmask,1) = tmp;
w_n = w_n | any(nmask);
nmask = X(:,4) < X(:,2); tmp = X(nmask,4); X(nmask,4) = X(nmask,2); X(nmask,2) = tmp;
w_n = w_n | any(nmask);
nmask = Y(:,3) < Y(:,1); tmp = Y(nmask,3); Y(nmask,3) = Y(nmask,1); Y(nmask,1) = tmp;
w_n = w_n | any(nmask);
nmask = Y(:,4) < Y(:,2); tmp = Y(nmask,4); Y(nmask,4) = Y(nmask,2); Y(nmask,2) = tmp;
w_n = w_n | any(nmask);
if w_n
    warning('negative-size bb is provided. bug?');
end

%%%% old idea. keep to avoid doing the same mistake again.
% to loop on the smaller matrix 
% BUT PROBLEM IS
% - can be slower!
% - wrong if some options like 'normalize wrt Y' is used!
%transposed = false;
%if size(X,1) > size(Y,1)
%    [Y X] = cn_idfun(X,Y); transposed = true;
%end

m = size(X,1);
n = size(Y,1);

%%%% extra functions
if isempty(option)
    NORM_OPT = 0; % default, 'union'
    IS_PIXELCOORD = true;
else
    % See cn_bboverlap_functions.cpp 
    switch(option)
        case 'union'
            NORM_OPT = 0; 
            IS_PIXELCOORD = true;
        case 'small'
            NORM_OPT = 1;
            IS_PIXELCOORD = true;
        case 'large'
            NORM_OPT = 2;
            IS_PIXELCOORD = true;
        case 'none'
            NORM_OPT = 3;
            IS_PIXELCOORD = true;
        case 'first'
            NORM_OPT = 4;
            IS_PIXELCOORD = true;
        case 'second'
            NORM_OPT = 5;
            IS_PIXELCOORD = true;

        case 'union+cont'
            NORM_OPT = 0; 
            IS_PIXELCOORD = false;
        case 'small+cont'
            NORM_OPT = 1;
            IS_PIXELCOORD = false;
        case 'large+cont'
            NORM_OPT = 2;
            IS_PIXELCOORD = false;
        case 'none+cont'
            NORM_OPT = 3;
            IS_PIXELCOORD = false;
       case 'first+cont'
            NORM_OPT = 4;
            IS_PIXELCOORD = false;
        case 'second+cont'
            NORM_OPT = 5;
            IS_PIXELCOORD = false;

        otherwise
            error('unknown option');
    end
end

if NORM_OPT ~= 3 
    % for 3 there is no normalization

    % calculate bb areas and also check bb validity
    tmp1 = Y(:,3)-Y(:,1); tmp2 = Y(:,4)-Y(:,2);
    if IS_PIXELCOORD
        tmp1 = tmp1 + 1; tmp2 = tmp2 + 1;
    end
    assert(all(tmp1>-eps)); assert(all(tmp2>-eps));
    areaY = tmp1 .* tmp2;

    tmp1 = X(:,3)-X(:,1); tmp2 = X(:,4)-X(:,2);
    if IS_PIXELCOORD
        tmp1 = tmp1 + 1; tmp2 = tmp2 + 1;
    end
    assert(all(tmp1>-eps)); assert(all(tmp2>-eps));
    areaX = tmp1 .* tmp2;
else
    areaX = zeros(m,0); % dummy
    areaY = zeros(n,0);
end

