

/**
 * Author: R. Gokberk Cinbis
 */



#include <stdio.h>
#include "math.h"
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "cn_matlabarrays.hpp"

// TODO Convert NormType into a type and make getbboverlap specialize at training time.

enum NormType { NORM_UNION = 0, NORM_SMALL, NORM_LARGE, NORM_NONE, NORM_X, NORM_Y };

// curx, cury:      double array of coordinates (x1,y1,x2,y2)
// areax,areay:     See cn_bboverlap_prepare()
// pixelcoord:      See cn_bboverlap_prepare()
// normopt:         See cn_bboverlap_prepare()
// iserr            Pointer to a boolean variable. If not NULL, set upon an error. Otherwise, simply returns NaN.
double getbboverlap(const double *curx, const double *cury,const double areax,const double areay,   \
        bool pixelcoord,const NormType normopt,bool *iserr=NULL) {


    // find intersection
    double intx = MIN(cury[2],curx[2]) - MAX(cury[0],curx[0]);
    double inty = MIN(cury[3],curx[3]) - MAX(cury[1],curx[1]);

    if (pixelcoord) {
        ++intx;
        ++inty;
    }

    // intersection is zero => any relation currently defined is zero
    if ((intx < 1e-15) || (inty < 1e-15))
        return 0;
    double rel = intx * inty;  // area of intersection

    // normalization
    switch(normopt) {
        case NORM_UNION:
            // areaunion
            rel /= (areax + areay - rel);
            break;
        case NORM_SMALL:
            // smaller object
            rel /= MIN(areax,areay);
            break;
        case NORM_LARGE:
            // larger
            rel /= MAX(areax,areay);
            break;
        case NORM_NONE:
            // do nothing
            break;
        case NORM_X:
            rel /= areax;
            break;
        case NORM_Y:
            rel /= areay;
            break;
        default:
            if (iserr!=NULL)
                *iserr = true;
            else
                printf("UNKNOWN normopt = %d \n",normopt);
            rel = NAN;
    }

    return rel;

}
