function intim = cn_integralimcreate(im)
% For already quantized n-dimensional data with weights, calculates the integral image.  Simply does cumulative sum operation.
% It may be more efficient to directly compute "intim" outside this function, via skipping the intermediate (m n) matrix.
%
% INPUT
% im: m x n x d1 x d2 x ... where each band belongs to a feature dimension.
%
% OUTPUT
% intim: (m+1) x (n+1) x d1 x d2 x ... matrix.
%
% SEE cn_integralimget, ndImIntegralImage (includes quantization)
%
% R.G.Cinbis Feb 2010

sz = size(im);

im = reshape(im,[sz(1) sz(2) prod(sz(3:end))]);

intim = zeros( [size(im,1)+1 size(im,2)+1 size(im,3)] ); % this is necessary if height or width is 1.

intim(2:end,2:end,:) = cumsum(cumsum(im,1),2);

intim = reshape(intim,[sz(1)+1 sz(2)+1 sz(3:end)]);


