function p = cn_softmax(p,d,w)
% p = cn_softmax(x,d,w)
%
% Softmax avoiding overflow.
%
% Computes 
%
% v = bsxfun( @rdivide, w.*exp(x), sum(w.*exp(x),d) )
%
% INPUT
% x     
% d     Dimension
% [w]   Either a matrix or a vector s.t. size(w,d)==size(p,d).
%
% OUTPUT
% p     Output posterior probabilities s.t. sum(p,d)==1.
% 
% REFERENCES
% - Based on code of Jakob Verbeek. 
% - http://lingpipe-blog.com/2009/03/17/softmax-without-overflow/
%
% x = rand(10,5); d = 1;
% bsxfun( @rdivide, exp(x), sum(exp(x),d) )
% cn_softmax(x,d)
%
% R.G.Cinbis Apr 1

% renamed x to p to do avoid copying input unnecessarily.

p = bsxfun(@minus,p,max(p,[],d)); 
p = exp(p);

if nargin > 2
    assert(size(w,d)==size(p,d));
    p = bsxfun(@times,p,w);
end

p = bsxfun(@times,p,1./sum(p,d)); % still sums to 1.


