function [Y,I] = cn_sorttopk(X,k,dim,mode)
% [Y,I] = cn_sorttopk(X,k,dim,mode)
%
% Sort and return top-k (highest or lowest) elements.
% This function is much more efficient when k<<N than the following:
%
%   % eg,dim=1
%   [Y,I] = sort(X,dim,mode);
%   Y = Y(1:k,:);
%   I = I(1:k,:);
%
% Just like sort(), equal-valued items preserve their original 
% order in the output. Therefore, their indices are returned
% in ascending order.
%
% INPUT
% X         Numeric array.
% k         Top-"k" constant.
% [dim]     Dimension to sort. Default is first-nonsingleton dimension.
% [mode]    'ascend' (default) | 'descend'
%
% OUTPUT
% Y         Sorted top-k elements. Has same size with X except that size(Y,dim)=k.
% I         Has same size with Y. Indices of top elemets. 
%
% R.G.Cinbis Jan 2012

% todo: change the naive implementation to median-of-median
% basedimplementation.

sz_x = size(X);

assert(isscalar(k));

if nargin < 3
    dim = find(sz_x > 1,1,'first');
    if isempty(dim)
        dim = 1;
    end
end

if nargin < 4
    isascend = true;
else
    switch(mode)
        case 'ascend'
            isascend = true;
        case 'descend'
            isascend = false;
        otherwise
            disp(mode);
            error('unknown mode');
    end
end

[Y,I] = cn_sorttopk_mex(X,k,dim,isascend);


