function splitind = cn_splitindices(numitem,numsplit)
% splitind = cn_splitindices(numitem,numsplit)
%
% Returns full indices 1...numitem split into numsplit sets.
% - Return numsplit-many non-empty splits. 
% - Larger sets are returned first.
% - Returns in increasing index order.
%
% INPUT
% numitem   Number of items (NOT A VECTOR).
% numsplit  Exact number of splits to get.
%
% OUTPUT
% splitind  (1 numsplit) Cell array each containing rowvec of indices.
%
% Example:
% splitind = cn_splitindices(10,8)
%
% SEE cn_vec2chunks 
%
% R.G.Cinbis, June 2010

assert(isscalar(numitem),'numitem should be scalar');

if numitem==0
    error('numitem==0 is an error');
end

if numsplit > numitem
    error('Too many splits.');
elseif numsplit < 1
    error('Invalid number of splits');
end

k = floor(numitem / numsplit);
m = numitem - k * numsplit; % < k
Ns = repmat(k,1,numsplit);
Ns(1:m) = Ns(1:m) + 1;
clear m k

splitind = cell(1,numsplit);
i1 = 1;
for j = 1:numsplit
    i2 = i1 + Ns(j) - 1;
    splitind{j} = i1:i2;
    i1 = i2 + 1;
end

cn_assert(i2 == numitem);

