function res = isfileordir(path)
% Check if a file or directory exists. Note that isfileordir(path)>0 is equivalent to "exists"
% check without using matlab search path.
% Follows symbolic links before returning the result.
%
% INPUT
% path      Just a path. Is not allowed to have wildcards.
%           To use with cell array, use cellfun(@isfileordir,paths). Cell
%           arrays are not supported to avoid mistakes.
%
% OUTPUT
% res       'f'     A file or a symlink to a file.
%           'd'     A directory or a symlink to a directory. Whether to give '/' at the end 
%                   doesnt matter. Behaviour is consistent with isdir() of matlab.
%           char(0) File/directory doesn't exist (or symlink is broken). Note that double(0)==char(0)==false
%
% R.G.Cinbis, Oct 2011

if iscell(path)
    error('Cell array of paths is not supported to avoid mistakes');
    %res = char(zeros(size(path)));
    %for i = 1:numel(path)
    %    res(i) = helper(path{i});
    %end
else
    res = helper(path);
end


function res = helper(path)

if isempty(path)
    path = pwd;
end

assert(~any(path=='*'),'wildcards in the path are not allowed');

% before calling dir(path), check if it is a directory.
if isdir(path)
    res = 'd';
else
    x = dir(path); % dont use exist() here, I find it less reliable.

    if isempty(x)
        res = char(0);
    elseif length(x) > 1
        error('bug!');
    else
        assert(~(x.isdir)); % sanity check
        res = 'f';
    end

end

%can be slow on large directories
%
%x = dir(path);
%if isempty(x)
%    res = char(0);
%elseif length(x) > 1
%    assert(x(1).name=='.'); % sanity check
%    res = 'd';
%else
%    assert(~(x(1).isdir)); % sanity check
%    res = 'f';
%end

