function [X, L, f_imi] = sample_features(imgroot, imnames, desc, maxsample, randseed0)

% Compute and randomly sample at most maxsample features of type desc from the images imnames
  
  % imnames = {imnames{1:20}}; 
  % max features per image
  maxperim = 2 * ceil( maxsample / length(imnames) );

  % 1st stage sampling
  
  fprintf('  Sampling training features (max %d per image)\n', maxperim);
  
  params = {};
  for i = 1:length(imnames)
    params{i} = {imgroot, imnames, desc, maxperim, randseed0, i};
  end
  
  feats = startmulticoremaster(@sample_image_features, params, struct('nrOfEvalsAtOnce', 10)); 
  feats = [feats{:}];
  % 2nd stage sampling
  
  for imi = length(imnames):-1:1
    nfeats(imi) = size(feats(imi).d_cols,2);
  end
  
  cn_setrandseed(randseed0);
  
  if sum(nfeats) > maxsample
    nfeats_org = nfeats;
    [nfeats, masks] = cn_subsamplesets(struct('nitems', nfeats,'maxtotal', maxsample));
    fprintf('Sampling from %d to %d features (maxsample=%d)\n',...
            sum(nfeats_org), sum(nfeats), maxsample);
    for imi = length(imnames):-1:1      
      feats(imi) = fv_util_maskfeats(feats(imi),masks{imi});
    end
  else
    fprintf('Collected %d samples.\n',sum(nfeats));
  end
  
  X = feats; 
  for j = length(X):-1:1 
    f_imi{j} = repmat(j,1,size(X(j).d_cols,2));
  end
  f_imi = [f_imi{:}]; 
    
  % cat then
  L = double([X.f_xynorm]); % (2 N)
  X = double([X.d_cols]); % (D N)
  assert(all(isfinite(L(:))));
  assert(all(isfinite(X(:)))); 
  
end


function feat = sample_image_features(imgroot, imnames, desc, maxperim, randseed0, i)
  % process one image
  
  filename = [imgroot char(imnames(i)) '.jpg'];    
          
  fprintf('   image %d/%d %s \n', i, length(imnames), filename);
        
  % compute the features
  [f_xynorm, d_cols] = extract_features(filename, desc, false);

  
  if size(d_cols, 2) > maxperim
    % subsample them
    cn_setrandseed(randseed0 + i);    
    m = cn_randmask(size(d_cols, 2), maxperim);    
    f_xynorm = f_xynorm(:, m);
    d_cols = d_cols(:, m);
  end

  feat = struct('f_xynorm', f_xynorm, 'd_cols', d_cols);   
end

