
% return the indices of the bboxes that are kept after non-max suppression

function selind = samples_nonmax_suppression(bbs, scores, maxov, minscoreth, maxnumdet)
  % remove boxes that overlap more than maxov in a greedy way (prefer remove low scores)
  [~, sort2org] = sort(scores, 'descend');
  [bbs,~,areas,~,NORM_OPT,IS_PIXELCOORD] = cn_bboverlap_prepare(bbs, zeros(0, 4), 'union');
  bbs = bbs.'; % (4 nbb)
  selind = cn_nmsgreedy_mex(bbs, areas, NORM_OPT, IS_PIXELCOORD, scores, sort2org, maxov, minscoreth, maxnumdet);
end


function [X Y areaX areaY NORM_OPT IS_PIXELCOORD] = cn_bboverlap_prepare(X,Y,option)

  if ~isa(X,'double') || ~isa(Y,'double')
    % annoying: disp('cn_bboverlap requires data in double, converting to double automatically');
    X = double(X);
    Y = double(Y);
  end
  
  % handle negative-sized bb's:
  w_n = false;
  nmask = X(:,3) < X(:,1); tmp = X(nmask,3); X(nmask,3) = X(nmask,1); X(nmask,1) = tmp;
  w_n = w_n | any(nmask);
  nmask = X(:,4) < X(:,2); tmp = X(nmask,4); X(nmask,4) = X(nmask,2); X(nmask,2) = tmp;
  w_n = w_n | any(nmask);
  nmask = Y(:,3) < Y(:,1); tmp = Y(nmask,3); Y(nmask,3) = Y(nmask,1); Y(nmask,1) = tmp;
  w_n = w_n | any(nmask);
  nmask = Y(:,4) < Y(:,2); tmp = Y(nmask,4); Y(nmask,4) = Y(nmask,2); Y(nmask,2) = tmp;
  w_n = w_n | any(nmask);
  if w_n
    warning('negative-size bb is provided. bug?');
  end
  
  
  m = size(X,1);
  n = size(Y,1);
  
  NORM_OPT = 0; 
  IS_PIXELCOORD = true;
  
  
  % calculate bb areas and also check bb validity
  tmp1 = Y(:,3)-Y(:,1); tmp2 = Y(:,4)-Y(:,2);
  if IS_PIXELCOORD
    tmp1 = tmp1 + 1; tmp2 = tmp2 + 1;
  end
  assert(all(tmp1>-eps)); assert(all(tmp2>-eps));
  areaY = tmp1 .* tmp2;
  
  tmp1 = X(:,3)-X(:,1); tmp2 = X(:,4)-X(:,2);
  if IS_PIXELCOORD
    tmp1 = tmp1 + 1; tmp2 = tmp2 + 1;
  end
  assert(all(tmp1>-eps)); assert(all(tmp2>-eps));
  areaX = tmp1 .* tmp2;
  
end
