  % adapted from Gokberk's  fv_candbox_tool_withmasks

function out = segments_to_masks( imsize, seg_filename, maskfile, randseed)

  cn_setrandseed(randseed);
  
  out = struct(); 
  out.imsize = int16(imsize);

  fprintf('  reading segments %s\n', seg_filename);
  
  x = load(seg_filename);
  trees = x.trees;

  maskout = [];

  % assert p.method = 'superpixelsinside'
  [out.bbs, maskout.masks] = method_superpixelsinside( trees );
  fprintf('    writing masks %s\n', maskfile);
  maskout.bbs = out.bbs;
  save(maskfile,'-v7','-struct','maskout');

end







function [bbs,masks] = method_superpixelsinside( trees )
  
  unqbbs = getboxes(trees);
  p.ifempty = 'skip'; 
  p.ovth = 1; 
  p.superpixelseg = 'avg';
  
  [bbs,masks] = fv_candbox_superpixelsinside( p, unqbbs, trees );
  [bbs,masks] = rmemptyboxes(bbs,masks);

  if isfield(p,'maskth')
    for i = 1:length(masks)
        masks{i} = masks{i} >= p.maskth;
    end
    % NO! [bbs,masks] = rmemptymasks(bbs,masks,1);
  end

end


function [bbs,masks] = rmemptyboxes(bbs,masks)
% remove boxes without any superpixels inside.

  assert(iscell(bbs));
  assert(iscell(masks));
  m = cellfun(@isempty,bbs); 
  bbs = bbs(~m);
  bbs = int16(vertcat( bbs{:} ));
  masks = colvec( masks(~m) );
  assert(length(masks)==size(bbs,1));
  
end



function unqbbs = getboxes(trees);
% define boxes for which we'll find masks.

  imsize = size(trees(1).leafmap);

  % switch(p.boxes) : case 'segments'
  % find segments that larger than the minimum required size
  segbbs = vertcat(trees.bbs); % (nseg 4)
  
  % old: m = min( segbbs(:,4)-segbbs(:,2)+1, segbbs(:,3)-segbbs(:,1)+1 ) >= p.minsz;
  segbbsuint8 = double(cn_mapptseasy('im->uint8',imsize,'bbs',segbbs)); 
  m = min( segbbsuint8(:,4)-segbbsuint8(:,2)+1, segbbsuint8(:,3)-segbbsuint8(:,1)+1 ) >= 10;
  segbbs = maskdim(segbbs,1,m);
  
  % find unique candidate boxes
  unqbbs = unique(detach(segbbs),'rows');

end

