
function unsupervised_training() 

  index = dataset_definition();
  
  % nb of box samples for training, set to Inf for non-toy dataset
  max_sample = 1000; 

  % make output directories
  mkdir data/feature_sample
  mkdir data/trained
  
  for desc = {'sift', 'color'}

    desc = char(desc);
    
    pca_file = ['data/trained/local_PCA_' desc '.mat'];
    
    if 1,         
      PCA = train_pca(index, desc, max_sample); 
      fprintf('Storing PCA matrix in %s\n', pca_file);  
      save(pca_file, '-v7', 'PCA');
    else 
      load(pca_file)   
    end

    gmm_file = ['data/trained/local_GMM_' desc '.mat'];
    if 1, 
      dict = train_gmm(index, desc, PCA, max_sample); 
      fprintf('Storing GMM dictionary in %s\n', gmm_file);  
      save(gmm_file, '-v7', 'dict');
    else 
      load(gmm_file);
    end

    % we use distinct product quantizers for Fisher vectors from boxes and from masks   
    
    for src = {'boxes', 'masks'}    
      src = char(src); 
      pq_file = [ 'data/trained/PQ_centroids_' desc '_for_' src '.mat']; 
      pq = train_pq(index, desc, src, PCA, dict, max_sample);     
      fprintf('Storing PQ centroids in %s\n', pq_file);  
      save(pq_file, '-v7', 'pq');
    end
        
  end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Learn local descriptor PCA

function PCA = train_pca(index, desc, max_sample)

  % select images from which to train
  train_imind = find(index.all_trainmask(1,:));
  train_imnames = index.imnames(train_imind);

  fprintf('  training PCA on %s descriptor from %d images\n', desc, length(train_imind));
  
  sample_fname = ['data/feature_sample/' desc '_for_pca.mat'];
  if 1
    randseed0 = 100;
    [X, L, f_imi] = sample_features(index.imgroot, train_imnames, desc, min(max_sample, 5e5), randseed0);
    fprintf('Storing %s samples for PCA training in %s\n', desc, sample_fname);
    save(sample_fname, 'X', 'L', 'f_imi');
  else
    fprintf('Loading %s samples for PCA training from %s\n', desc, sample_fname);
    load(sample_fname);
  end
  
  % Train local descriptor PCA (to 64 dim)
  pca_ndim = 64;
  % px = struct('method', 'pca', 'ndim', pca_ndim, 'unitsc', 0); 
  dimred = train_PCA(X, pca_ndim);
  PCA = struct(); 
  PCA.mu = dimred.data.mu; 
  PCA.U = dimred.data.U; 
  PCA.ndim = pca_ndim;   
  
end


function dimred =  train_PCA(X, ndim)
 
  D = size(X,1);
  N = size(X,2);

  K = 1;
  islocal = false;
  % piotr's PCA implementation.
  % it internally does random subsampling. I think it is better to 
  % use pcag in general for our purposes even for global pca.

  assert(~isempty(strfind(help('pca'),'Piotr Dollar')),'Requires Piotr Dollar''s PCA implementation to be in the path');
  [U,dimred.data.mu,vars] = pca(X);
  assert(all(diff(vars)<=0)); % make sure already sorted.

  dimred.data.vars = vars; % just for information.
  dimred.data.U = U;
  assert(size(dimred.data.U,1)==D);
  assert(length(dimred.data.mu)==D);
  
  if length(vars) < ndim
    dimred.ndim = length(vars);
    mywarn(sprintf('ndim option is reduced from %d -> %d \n',p.ndim,dimred.ndim));
  end
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Learn local descriptor GMM

function dict = train_gmm(index, desc, PCA, max_sample)

  K = 64; % nb of Gaussians

  % select images from which to train
  train_imind = find(index.all_trainmask(1,:));
  train_imnames = index.imnames(train_imind);
 
  sample_fname = ['data/feature_sample/' desc '_for_gmm.mat' ];  
  if 1
    randseed0 = 2;
    [X, L, f_imi] = sample_features(index.imgroot, train_imnames, desc, min(max_sample, 15e5), randseed0);
    fprintf('Storing sift samples for GMM training in %s\n', sample_fname);
    save(sample_fname, 'X', 'L', 'f_imi');
  else
    fprintf('Loading sift samples for PCA training from %s\n', sample_fname);
    load(sample_fname);
  end
  
  
  % reduce to 64 dim
  X = pcaApply(X, PCA.U, PCA.mu, PCA.ndim);

  % train GMM
  ops = struct(); 
  ops.eps = 1e-5; 
  ops.dim = 0; % 0. factor dim
  ops.iso = 0; % 0. isotropic noise -- Psi columns are constant (within cmponents)
  ops.eqn = 0; % 0. equal noise -- Psi rows are constant. (within dimension)
  ops.dia = 1; % verbosity
  ops.mxi = 100; % max iter (100)
  ops.ini = 100; % 
  ops.minvar = var(X,0,2) .* 0.01;
  ops.sparsityk = Inf; 
  ops.maxbyte = 5e9;
  
  cn_setrandseed(101);
  
  [MoFA_init A_init]= init_mfa(X, K, ops);
  [LogL, MoFA, post] = mfa(X, K, ops, MoFA_init); 
    
  % convert to standard format

  % MoFA.mix  (K 1)
  % MoFA.Psi  (D K) variances
  % MoFA.M    (D K) means
  assert(isempty(MoFA.W));  % ie latent d==0

  %   .K          Number of components.
  %   .data.mix   (1 K) Mixing weights.
  %   .data.mu    (D K) Means
  %   .data.vars  (D K) Variances
  dict = struct(); 
  dict.K = K;
  dict.mix = reshape(MoFA.mix,1,dict.K);
  dict.mu = MoFA.M;
  dict.vars = MoFA.Psi;
  
  assert(all(isfinite(dict.vars(:))))
  assert(all(isfinite(dict.mu(:))))

  % precompute some stuff
  dict_prep = cn_gaussdiag_precalc(struct('mu',dict.mu, 'vars', dict.vars, 'priors', dict.mix));
  dict.D = dict_prep.D; 
  dict.w_x = dict_prep.w_x; 
  dict.w_x2 = dict_prep.w_x2; 
  dict.logprior = dict_prep.logprior; 
  dict.bias = dict_prep.bias; 
  
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Learn PQ on FVs

function pq = train_pq(index, desc, src, PCA, dict, max_sample)
  
  dlen = 2 * dict.K * PCA.ndim + dict.K; % local descriptor length
  ds = 8;                      % dimension of subvectors
  nsq = dlen / ds;
  
  cn_setrandseed(101);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Get training data 
  
  % load candidate bboxes
  
  x = load('data/masks/bbs.mat'); 
  candbbs = x.boxes;
    
  ind = 1:length(candbbs);
  
  % add some info
  samples = struct();  
  train_images = find(index.all_trainmask); 
  for j = 1:length(train_images)
    imi = train_images(j);
    samples(j).orgimi = imi;
    samples(j).bbs = int16(candbbs(imi).bbs);
    samples(j).imsize = double(index.imsizes(imi, :)); % for simplicity, fix this to double.
    samples(j).bbs = candbbs(imi).bbs;
    if isequal(src, 'boxes')
      % also used full image as training source (M12: not useful, also done below)
      imsize = index.imsizes(imi, :);
      bb = int16([1 1 imsize([2 1])]);
      samples(j).bbs = [bb; samples(j).bbs]; 
    end
  end
  clear candbbs;   
    
  % subsample windows
  x = {}; y = {}; orgn = {};
  for i = length(samples):-1:1
    n = size(samples(i).bbs,1);
    x{i} = repmat(i,1,n);
    y{i} = 1:n;
  end
  x = [x{:}];
  y = [y{:}];
  nsample  = min(5e4, max_sample);  
  m = cn_randmask(length(y), min(length(y), nsample));
  x = x(m);
  y = y(m);
  
  params = {};  
  for i = 1:length(samples)
    imi = samples(i).orgimi;
    orgbbind = y(x==i); % maps to global bb indices
    params{i} = {index.imgroot, desc, src, PCA, dict, index.imnames{imi}, index.imsizes(imi, :), ...
                 samples(i).bbs(orgbbind,:), orgbbind }; 
  end

  fprintf('  Sampling %d FVs from %d images\n', nsample, length(samples));

  data = startmulticoremaster(@sample_fv_from_image, params, struct('nrOfEvalsAtOnce', 10));  
  data = [data{:}];   

  if 0    % too bulky...
    sample_fname = ['data/feature_sample/FV_' desc '_' src '_for_pq.mat' ];  
    fprintf('Storing FV samples for PQ training in %s\n', sample_fname);
    save(sample_fname, 'data', '-v7.3');
  end

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Train PQ (adapted from pq_new)

  fprintf('  Training Product Quantizer on %d FVs in %d D\n', size(data, 2), size(data, 1));
  
  nsqbits = 8;         % the number of subquantizers is fixed to 8-bits in this implementation
  p = struct();
  p.data = data';
  p.nsq = nsq; 
  p.K = 256;           % number of centroids per subquantizer
  p.randseed = 101;
  p.maxiter = 100;
  M = cn_kmeansparallel(p);

  pq = struct();
  pq.ks = 256;
  pq.ds = ds;
  pq.nsq = nsq;
  pq.centroids = {}; 
  for z = 1:nsq
    pq.centroids{z} = single(M(:,:,z));
  end
  % precompute some stuff
  pq = pq_prep(pq);
  
end


function image_fvs = sample_fv_from_image(imgroot, desc, src, PCA, dict, imname, imsize, bbs, orgbbind)
 
  % image data
  img_filename = [imgroot imname '.jpg'];    

  grids = {[1 1]}; % no grid for PQ training
    
  % no additional safety check is necessary since masks will be required to have the same
  % size as the boxes. if there is an indexing error, it will be apparent.
  
  if isequal(src, 'boxes')
    % also used full image as training source
    % add full images as additional samples for pq training     
    bb = int16([1 1 imsize([2 1])]);    
    bbs = unique([bb; bbs],'rows');      
    masks = [];
  end
  
  if isequal(src, 'masks')
    masks_filename = ['data/masks/' imname '.mat'];    
    maskout = load(masks_filename);
    
    % remap masks to original intial boxes
    masks = maskout.masks(orgbbind);
  end
  
  [per_cell_descs, per_cell_npts] = fv_from_samples(img_filename, imsize, ...
                                                    desc, bbs, PCA, dict, grids, masks);
  
  image_fvs = per_cell_descs{1};     
end
